@@ -38,6 +38,7 @@ class MicroscopeClient(object):
         else:
             self.host = host
             self.port = port
+            self.get_json(self.base_uri)
         logging.info(f"Connecting to microscope {self.host}:{self.port}")
         self.populate_extensions()
 
@@ -114,17 +115,27 @@ class MicroscopeClient(object):
         r.raise_for_status()
         return r.json()
 
-    def capture_image(self, ):
+    def capture_image(self):
         """Capture an image and return it as a PIL image object"""
         payload = {
             "use_video_port": True,
             "bayer": False,
         }
-        r = requests.post(self.base_uri + "/actions/camera/ram-capture", json=payload)
+        r = requests.post(self.base_uri + "/actions/camera/ram-capture", json=payload, headers={'Accept': 'image/jpeg'})
         r.raise_for_status()
         image = PIL.Image.open(io.BytesIO(r.content))
         return image
 
+    def capture_image_to_disk(self):
+        """Capture an image and save it to disk"""
+        payload = {
+            "use_video_port": True,
+            "bayer": False,
+            "temporary" :False
+        }
+        r = requests.post(self.base_uri + "/actions/camera/capture", json=payload)
+        return r
+
     def grab_image(self):
         """Grab an image from the stream and return as a PIL image object"""
         r = requests.get(self.base_uri + "/streams/snapshot")
@@ -132,6 +143,30 @@ class MicroscopeClient(object):
         image = PIL.Image.open(io.BytesIO(r.content))
         return image
 
+    def grab_mjpeg(self, timeout, output=None, stop_event=None, max_size=None):
+        """Capture the mjpeg stream from the camera"""
+        if output is None:
+            output = io.BytesIO()
+        logging.debug("connecting to MJPEG stream")
+        r = requests.get(self.base_uri + "/streams/mjpeg", stream=True)
+        logging.debug("MJPEG request made")
+        r.raise_for_status()
+        logging.debug("Stream opened OK")
+
+        start_time = time.time()
+        received_bytes = 0
+        for chunk in r.iter_content(chunk_size=1024):
+            output.write(chunk)
+            received_bytes += len(chunk)
+            if timeout is not None and time.time() - start_time > timeout:
+                break
+            if stop_event is not None and stop_event.is_set():
+                break
+            if max_size is not None and received_bytes > max_size:
+                break
+        return output.getvalue()
+
+
     def grab_image_array(self, ):
         """Grab an image and return it as a numpy ndarray"""
         return np.array(self.grab_image())
@@ -148,6 +183,11 @@ class MicroscopeClient(object):
         """Move the stage up and down, and pick the sharpest position."""
         return self.extensions["org.openflexure.autofocus"]["fast_autofocus"].post_json()
 
+    @property
+    def configuration(self):
+        """Return the position of the stage as a dictionary"""
+        return self.get_json("/instrument/configuration/")
+
 class MicroscopeExtension():
     """A class that represents a microscope extension"""
     def __init__(self, extension_dict):
@@ -260,4 +300,25 @@ def find_first_microscope(timeout=10):
     microscopes = find_microscopes(timeout, 1)
     if len(microscopes) == 0:
         raise Exception("There are no microscopes advertised on the local network")
-    return MicroscopeClient(microscopes[0])
\ No newline at end of file
+    return MicroscopeClient(microscopes[0])
+
+def iterate_mjpeg_frames_raw(buffer):
+    """Given a captured MJPEG stream, iterate through the frames."""
+    while True:
+        start = buffer.find(b'\xff\xd8')
+        if start > 0:
+            logging.debug(f"Found {start} bytes of junk between JPEG frames")
+        end = buffer.find(b'\xff\xd9')
+        if start != -1 and end != -1:
+            jpg = buffer[start:end+2]
+            buffer = buffer[end+2:]
+            logging.debug(f"Got a JPEG frame with {len(jpg)} bytes")
+            yield jpg
+        else:
+            logging.debug(f"There were {len(buffer)} bytes left over from the MJPEG stream")
+            return
+
+def iterate_mjpeg_images(buffer):
+    for frame in iterate_mjpeg_frames_raw(buffer):
+        yield PIL.Image.open(io.BytesIO(frame))
+
